@@ -1,7 +1,6 @@
 """
 Wraps scipy.optimize.fmin_* algorithms using Criterion instances.
 """
-from copy import copy
 import numpy as np
 import scipy.optimize as opt
 
@@ -55,7 +54,7 @@ class FminWrapper(object):
         if x0 is None:
             self.current_solution = np.zeros(self.shapein)
         else:
-            self.current_solution = copy(x0)
+            self.current_solution = np.array(x0)
 
 class FminCOBYLA(FminWrapper):
     __doc__ = FminWrapper.__doc__ + opt.fmin_cobyla.__doc__
