@@ -10,7 +10,7 @@ from .core import (Operator, BlockRowOperator, BroadcastingOperator,
                    CompositionOperator, DenseOperator, DiagonalOperator,
                    ReductionOperator, DirectOperatorFactory,
                    ReverseOperatorFactory, Variable, X)
-from .utils import cast, ifind, tointtuple
+from .utils import cast, ifirst, tointtuple
 
 __all__ = [
     'BandOperator',
@@ -168,7 +168,7 @@ class EinsteinSumOperator(Operator):
             raise ValueError("The number of input operands '{0}' does not match"
                              " that from the subscripts '{1}'.".format(len(
                              operands), len(ins)))
-        self.iX = ifind(operands, lambda x: x is X)
+        self.iX = ifirst(operands, lambda x: x is X)
         self.operands = operands
         self.subscripts = subscripts
         self.lsubscripts = tuple([self._get_subscript_as_list(s)
